
# Welcome to your CDK Python project, made with Halloumi CDK!

You should explore the contents of this repository. Executing its python will create a cloudformation
Template for two stacks: a stack named after your project, and a skeleton stack, which is nested in
the project stack and creates a public subnet and flow logs. 

The `cdk.json` file tells the CDK Toolkit how to execute your app.

This project is set up like a standard Python project.  The initialization process also creates
a virtualenv within this project, stored under the .env directory.  To create the virtualenv 
it assumes that there is a `python3` executable in your path with access to the `venv` package.
If for any reason the automatic creation of the virtualenv fails, you can create the virtualenv
manually once the init process completes.

To manually create a virtualenv on MacOS and Linux:

```
$ python3 -m venv .env
```

After the init process completes and the virtualenv is created, you can use the following
step to activate your virtualenv.

```
$ source .env/bin/activate
```

If you are a Windows platform, you would activate the virtualenv like this:

```
% .env\Scripts\activate.bat
```

Once the virtualenv is activated, you can install the required dependencies.

```
$ pip install -r requirements.txt
```

At this point you can now synthesize the CloudFormation template for this code.

```
$ cdk synth
```

You can now begin exploring the source code, contained in the hello directory.
There is also a very trivial test included that can be run like this:

```
$ pytest
```

To add additional dependencies, for example other CDK libraries, just add to
your requirements.txt file and rerun the `pip install -r requirements.txt`
command.

Deploy everything as follows

```
$ cdk deploy "*"
```

and without interruptions (required for ci/cd)

```
$ cdk deploy "*" --require-approval never
```

# Compile using docker

If you already have a locksmith session or you are using a CI/CD tool that already 
has aws credentials, then use the `cdk.cicd.Dockerfile`

```
> docker build . -f cdk.cicd.Dockerfile --tag halloumi-cdk:<version>

e.g. 

$ docker build . -f cdk.cicd.Dockerfile --tag halloumi-cdk:1.10

$ docker run -v (pwd):/project halloumi-cdk:1.10

```

If you want to work inside a docker container, then use the `cdk.locksmith.Dockerfile`

```
> docker build . -f cdk.locksmith.Dockerfile --tag halloumi-locksmith-cdk:<version>

e.g. 

$ docker build . -f cdk.locksmith.Dockerfile --tag halloumi-locksmith-cdk:1.10

$ docker run -v (pwd):/project -v ~/.aws/credentials:/root/.aws/credentials -it halloumi-locksmith-cdk:1.10

bash-4.2# cd /project

bash-4.2# pip install -r requirements.txt

bash-4.2# cdk synth
```

# Useful commands

 * `cdk ls`          list all stacks in the app
 * `cdk synth`       emits the synthesized CloudFormation template
 * `cdk deploy`      deploy this stack to your default AWS account/region
 * `cdk diff`        compare deployed stack with current state
 * `cdk docs`        open CDK documentation

Enjoy!
