from aws_cdk import (
    aws_cloudformation as cfn,
    aws_ec2 as ec2,
    aws_iam as iam,
    aws_logs as logs,
    core
)


class SkeletonStack(cfn.NestedStack):

    def __init__(self, scope: core.Construct, id: str,
                 config: dict, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        self.vpc = ec2.Vpc(
            self, f'{config["stack_name"]}Vpc',
            cidr=config['cidr_block'],
            max_azs=99,  # use all available AZs,
            subnet_configuration=[
                {
                    'cidrMask': 28,
                    'name': 'nat',
                    'subnetType': ec2.SubnetType.PUBLIC
                }
            ]
        )

        # Creation of the Log Group
        flowlogs_group_id = f'{config["stack_name"]}VpcFlowLogGroup'
        flowlogs_group = logs.CfnLogGroup(
            self, flowlogs_group_id
        )

        # IAM Role for flow logs
        flowlogs_iam_role_id = f'{config["stack_name"]}FlowlogsIamRole'
        flowlogs_iam_role = iam.Role(
            self, flowlogs_iam_role_id,
            assumed_by=iam.ServicePrincipal('vpc-flow-logs.amazonaws.com')
        )

        # Adding the policy to the IAM Role
        flowlogs_iam_role.add_to_policy(
            iam.PolicyStatement(
                actions=[
                    'logs:CreateLogGroup',
                    'logs:CreateLogStream',
                    'logs:PutLogEvents',
                    'logs:DescribeLogGroups',
                    'logs:DescribeLogStreams'
                ],
                effect=iam.Effect.ALLOW,
                resources=["*"]
            )
        )

        # Creation of the VPC Flow Log
        FlowLogs_id = f'{config["stack_name"]}VpcFlowLogs'
        ec2.CfnFlowLog(
            self, FlowLogs_id,
            resource_id=self.vpc.vpc_id,
            resource_type='VPC',
            traffic_type='REJECT',
            deliver_logs_permission_arn=flowlogs_iam_role.role_arn,
            log_destination_type='cloud-watch-logs',
            log_group_name=flowlogs_group.ref
        )

    def ref_vpc(self):
        return self.vpc
